#include <stm32f10x_lib.h>



#define AdresseExpanseur1 0x40	/*Adresse du premier expanseur de port I2C, de type  PCF8574 */
#define AdresseExpanseur2 0x42	/*Adresse du deuxime expanseur de port I2C, de type  PCF8574 */



/*Fonction qui sert  faire perdre son temps au STM32F100RB

Paramtre d'entre :

t - Valeur qui est proportionnelle au dlai que l'on souhaite obtenir.

Valeur de retour :

Aucune.

*/
void Delai(int t)
	{
		int i;
		for(i = 0; i < (10 * t); i++);
	}



/*Initialisation du module  I2C 1  du STM32F100RB en tant que  Matre 

Paramtre d'entre :

Aucun.

Valeur de retour :

Aucune.

*/
void Initialisation_I2C_1(void)
	{
		/* GPIOB clock enable */
		RCC->APB2ENR |= (1 << 3);

        /* I2C1 clock enable */
		RCC->APB1ENR |= (1 << 21);

        /* I2C1 SDA and SCL configuration */
		GPIOB->CRL = ((GPIOB->CRL & 0x00FFFFFF) | (0xFF000000));

        /* Enable I2C1 reset state */
		RCC->APB1RSTR |= (1 << 21);
        /* Release I2C1 from reset state */
		RCC->APB1RSTR &= ~(1 << 21);

		I2C1->CR1 = 0x0401;
		I2C1->CR2 = 0x0108;
		
		I2C1->SR1 = 0x0000;
		I2C1->SR2 = 0x0000;
		
		I2C1->CCR = 0x8006;
		
		I2C1->OAR1 = 0x4028;
		I2C1->OAR2 = 0x0000;
		
		I2C1->TRISE = 0x0003;
	}



/*Lecture d'un ou de plusieurs octets grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

adresse - Adresse de lecture, sur le bus  I2C 1  du composant que l'on souhaite lire,
nombre - Nombre d'octets  lire,
receptacle[] - Tableau qui est destin  contenir le ou les octets qui auront t lus.

Valeur de retour :

1 si la lecture s'est effectue correctement, -1 s'il y a eu une erreur lors de la lecture.

*/
int LirePlusieursOctets(unsigned char adresse, unsigned char nombre, unsigned char receptacle[])
	{
	    int temp = 0;
	    int Timeout = 0;
			if(nombre <= 2)
		  		{
		           /* Set POS bit */
		            I2C1->CR1 |= 0x0800;
		            Timeout = 0xFFFF;
		            /* Send START condition */
		            I2C1->CR1 |= 0x0100;
		            /* Wait until SB flag is set: EV5 */
			            while ((I2C1->SR1&0x0001) != 0x0001)
				            {
				                if (Timeout-- == 0)
				                    return (-1);
				            }
		            Timeout = 0xFFFF;
		            /* Send the slave address */
		            I2C1->DR = adresse;
		            /* Wait until ADDR is set: EV6 */
			            while((I2C1->SR1&0x0002) != 0x0002)
				            {
				                if(Timeout-- == 0)
				                    return (-1);
				            }
		            /* Clear ADDR by reading SR2 register  */
		            temp = I2C1->SR2;
		            /* Clear ACK */
		            I2C1->CR1 &= 0xFBFF;
		            /* Wait until BTF is set */
		            	while((I2C1->SR1 & 0x00004) != 0x000004);
		            /* Program the STOP */
		            I2C1->CR1 |= 0x0200;
		            /* Read first data */
		            receptacle[0] = I2C1->DR;
						if(nombre == 2)
							{
					            /* Read second data */
					            receptacle[1] = I2C1->DR;
							}
			            /* Make sure that the STOP bit is cleared by Hardware before CR1 write access */
			            while ((I2C1->CR1&0x200) == 0x200);
		            /* Enable Acknowledgement to be ready for another reception */
		            I2C1->CR1  |= 0x0400;
		            /* Clear POS bit */
		            I2C1->CR1  &= 0xF7FF;
				}
			return 1;
	}




/*Lecture d'un octet grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

adresse - Adresse de lecture, sur le bus  I2C 1 , du composant que l'on souhaite lire.

Valeur de retour :

La donne qui a t lue si la lecture s'est effectue correctement, -1 s'il y a eu une erreur lors de la lecture.

*/
int Lire(unsigned char adresse)
	{
	 	unsigned char tampon[1];
			if(LirePlusieursOctets(adresse, 1, tampon) == (-1))
				{
					return (-1);
				}
		return tampon[0];
	}



/*Ecriture d'un ou de plusieurs octets grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

adresse - Adresse d'criture, sur le bus  I2C 1 , du composant que l'on souhaite crire,
nombre - Nombre d'octets  crire,
donnees[] - Tableau qui contient le ou les octets  crire.

Valeur de retour :

1 si l'criture s'est effectue correctement, -1 s'il y a eu une erreur lors de l'criture.

*/
int EcrirePlusieursOctets(unsigned char adresse, unsigned char nombre, unsigned char donnees[]) 
	{
		int temp = 0;
	    int Timeout = 0;
		int p = 0;
        Timeout = 0xFFFF;
        /* Send START condition */
        I2C1->CR1 |= 0x0100;
	        /* Wait until SB flag is set: EV5 */
	        while ((I2C1->SR1&0x0001) != 0x0001)
		        {
		            if (Timeout-- == 0)
						{
		 					Initialisation_I2C_1();
			                	return (-1);
						}
		        }
        /* Send the slave address */
        I2C1->DR = adresse;
        Timeout = 0xFFFF;
	        /* Wait until ADDR is set: EV6 */
	        while ((I2C1->SR1 &0x0002) != 0x0002)
		        {
		            if (Timeout-- == 0)
						{
		 					Initialisation_I2C_1();
			                	return (-1);
						}
		        }
        /* Clear ADDR flag by reading SR2 register */
        temp = I2C1->SR2;
        /* Write the first data in DR register (EV8_1) */
        I2C1->DR = donnees[0];
        /* Increment */
        p++;
        /* Decrement the number of bytes to be written */
        nombre--;
	        /* While there is data to be written */
	        while(nombre--)
		        {
		            /* Poll on BTF to receive data because in polling mode we can not guarantee the
		              EV8 software sequence is managed before the current byte transfer completes */
		            while ((I2C1->SR1 & 0x00004) != 0x000004);
			            /* Send the current byte */
			            I2C1->DR = donnees[p];
			            /* Point to the next byte to be written */
			            p++;
		        }
	        /* EV8_2: Wait until BTF is set before programming the STOP */
	        while ((I2C1->SR1 & 0x00004) != 0x000004);
	        /* Send STOP condition */
	    I2C1->CR1 |= 0x0200;
	        /* Make sure that the STOP bit is cleared by Hardware */
	        while ((I2C1->CR1&0x200) == 0x200);

	    	return 1;
	}



/*Ecriture d'un octet grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

adresse - Adresse d'criture, sur le bus  I2C 1 , du composant que l'on souhaite crire,
donnees - Donne  crire.

Valeur de retour :

1 si l'criture s'est effectue correctement, -1 s'il y a eu une erreur lors de l'criture.

*/
int Ecrire(unsigned char adresse, unsigned char donnee)
	{
	 	unsigned char tampon[1];
			tampon[0] = donnee;
		return (EcrirePlusieursOctets(adresse, 1, tampon));
	}



/*Fonction principale 

Paramtre d'entre :

Aucun.

Valeur de retour :

Aucune.

*/
int main (void)
	{
		int k;
		int r;

		int transfert;

    	Initialisation_I2C_1();
			
		Ecrire(AdresseExpanseur1, 0xFF);	/*Mise de toutes les broches du premier expanseur de port   1 */
		Ecrire(AdresseExpanseur2, 0xFF);	/*Mise de toutes les broches du deuxime expanseur de port   1 */

		Delai(10000);

		transfert = Lire((AdresseExpanseur1 | 0x01));	/*Lecture de l'tat des 8 broches du premier expanseur de port*/

		Ecrire(AdresseExpanseur2, transfert);	/*Les broches du deuxime expanseur de port prennent l'tat de celles du premier*/

		Delai(10000);
			 
			while(1)
				{
					for(k = 0; k < 7; k++)
						{
							Ecrire(AdresseExpanseur1, ~(1 << k));	/*Chenillard sur le premier expanseur de port*/
							Delai(10000);
						}
					Ecrire(AdresseExpanseur1, 0xFF);	/*Mise de toutes les broches du premier expanseur de port   1 */
					for(k = 0; k < 7; k++)
						{
							
							Ecrire(AdresseExpanseur2, ~(1 << k));	/*Chenillard sur le deuxime expanseur de port*/
							Delai(10000);
						}
					Ecrire(AdresseExpanseur2, 0xFF);	/*Mise de toutes les broches du deuxime expanseur de port   1 */
					for(r = 0; r < 15; r++)
						{
							Delai(10000);
						}
					Ecrire(AdresseExpanseur1, 0xAA);	/*Mise d'une broche sur deux du premier expanseur de port   1 */
					Ecrire(AdresseExpanseur2, 0xAA);	/*Mise d'une broche sur deux du deuxime expanseur de port   1 */
					for(r = 0; r < 15; r++)
						{
							Delai(10000);
						}
					Ecrire(AdresseExpanseur1, 0xFF);	/*Mise de toutes les broches du premier expanseur de port   1 */
					Ecrire(AdresseExpanseur2, 0xFF);	/*Mise de toutes les broches du deuxime expanseur de port   1 */
					for(r = 0; r < 15; r++)
						{
							Delai(10000);
						}
				}
	}



/*Fonction qui permet l'initialisation du microcontrleur STM32F100RB

Paramtre d'entre :

Aucun.

Valeur de retour :

Aucune.

*/
void SystemInit (void)
	{
	
	}
